package routes_test

import (
	"gitea.deepak.science/deepak/gogmagog/models"
	"gitea.deepak.science/deepak/gogmagog/routes"
	"github.com/stretchr/testify/assert"
	"net/http"
	"net/http/httptest"
	"strings"
	"testing"
	"time"
)

func TestEmptyPlans(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	router := routes.NewRouter(m)
	req, _ := http.NewRequest("GET", "/plans", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusOK, status)
	expected := `[]`

	assert.JSONEq(expected, rr.Body.String())
	contentType := rr.Header().Get("Content-Type")
	assert.Equal("application/json", contentType)
}

func TestOnePlan(t *testing.T) {
	// set up
	assert := assert.New(t)
	planDate, _ := time.Parse("2006-01-02", "2021-01-01")
	p := &models.Plan{PlanID: 6, PlanDate: planDate}
	m := getModel([]*models.Plan{p}, []*models.Action{})
	router := routes.NewRouter(m)
	req, _ := http.NewRequest("GET", "/plans", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusOK, status)
	// We pass in the date as a time.time so it makes sense that it comes back with a midnight timestamp.
	expected := `[
		{
			"plan_id": 6,
			"plan_date": "2021-01-01T00:00:00Z"
		}
	]`
	assert.JSONEq(expected, rr.Body.String())
	contentType := rr.Header().Get("Content-Type")
	assert.Equal("application/json", contentType)
}

func TestErrorPlan(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getErrorModel("Model always errors")

	router := routes.NewRouter(m)
	req, _ := http.NewRequest("GET", "/plans", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)
	// We pass in the date as a time.time so it makes sense that it comes back with a midnight timestamp.
	expected := `Internal Server Error`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestEmptyPlanErrorWriter(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getEmptyModel()

	router := routes.NewRouter(m)
	req, _ := http.NewRequest("GET", "/plans", nil)

	rr := NewBadWriter()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)

}
