package routes_test

import (
	"fmt"
	"gitea.deepak.science/deepak/gogmagog/models"
)

type multiStore struct {
	actions []*models.Action
	plans   []*models.Plan
}

func (ms *multiStore) SelectActions() ([]*models.Action, error) {
	return ms.actions, nil
}

func (ms *multiStore) SelectActionByID(id int) (*models.Action, error) {
	return ms.actions[0], nil
}

func (ms *multiStore) SelectPlans() ([]*models.Plan, error) {
	return ms.plans, nil
}

func (ms *multiStore) SelectPlanByID(id int) (*models.Plan, error) {
	return ms.plans[0], nil
}

func (ms *multiStore) InsertPlan(plan *models.Plan) (int, error) {
	return int(plan.PlanID), nil
}

func (ms *multiStore) SelectActionsByPlanID(plan *models.Plan) ([]*models.Action, error) {
	return ms.actions, nil
}

func (ms *multiStore) ConnectionLive() error {
	return nil
}

func getEmptyModel() *models.Model {
	ss := &multiStore{
		[]*models.Action{},
		[]*models.Plan{},
	}
	m := models.New(ss)
	return m
}
func getModel(plns []*models.Plan, acts []*models.Action) *models.Model {
	ss := &multiStore{
		actions: acts,
		plans:   plns,
	}
	m := models.New(ss)
	return m
}

func (e *errorStore) SelectActions() ([]*models.Action, error) {
	return nil, e.error
}

func (e *errorStore) SelectActionByID(id int) (*models.Action, error) {
	return nil, e.error
}

func (e *errorStore) SelectPlans() ([]*models.Plan, error) {
	return nil, e.error
}

func (e *errorStore) SelectPlanByID(id int) (*models.Plan, error) {
	return nil, e.error
}

func (e *errorStore) InsertPlan(plan *models.Plan) (int, error) {
	return 0, e.error
}

func (e *errorStore) SelectActionsByPlanID(plan *models.Plan) ([]*models.Action, error) {
	return nil, e.error
}

func (e *errorStore) ConnectionLive() error {
	return e.error
}

type errorStore struct {
	error error
}

func getErrorModel(errorMsg string) *models.Model {
	e := &errorStore{error: fmt.Errorf(errorMsg)}
	return models.New(e)
}
